const db = getDb("cache.sqlite");

db.table(
  "permissions",
  /* SQL */ `
  CREATE TABLE IF NOT EXISTS permissions (
    prefix TEXT PRIMARY KEY,
    allow INTEGER NOT NULL
  );
`,
);
export class FilePermissions {
  prefix!: string;
  /** Currently set to 1 always */
  allow!: number;

  // -- static ops --
  static getByPrefix(filePath: string): number {
    return getByPrefixQuery.get(filePath)?.allow ?? 0;
  }

  static getExact(filePath: string): number {
    return getExactQuery.get(filePath)?.allow ?? 0;
  }

  static setPermissions(prefix: string, allow: number) {
    if (allow) {
      insertQuery.run({ prefix, allow });
    } else {
      deleteQuery.run(prefix);
    }
  }
}

const getByPrefixQuery = db.prepare<
  [prefix: string],
  Pick<FilePermissions, "allow">
>(/* SQL */ `
  SELECT allow 
  FROM permissions 
  WHERE ? GLOB prefix || '*'
  ORDER BY LENGTH(prefix) DESC 
  LIMIT 1;
`);
const getExactQuery = db.prepare<
  [file: string],
  Pick<FilePermissions, "allow">
>(/* SQL */ `
  SELECT allow FROM permissions WHERE ? == prefix
`);

const insertQuery = db.prepare<[{ prefix: string; allow: number }]>(/* SQL */ `
  REPLACE INTO permissions(prefix, allow) VALUES($prefix, $allow);
`);
const deleteQuery = db.prepare<[file: string]>(/* SQL */ `
  DELETE FROM permissions WHERE prefix = ?;
`);

import { getDb } from "#sitegen/sqlite";
