import "./PhotoGrid.css";
import { MediaFile } from "@/file-viewer/models/MediaFile.ts";
const { isArray } = Array;

export interface PhotoGridProps {
  style: any;
  base: string;
  width: number;
  widths?: string[];
  heights?: number[];
  items: Item[];
}

type Item = [img: string, options?: ItemOptions];

interface ItemOptions {
  w?: number;
  h?: number;
  align?: keyof typeof alignments;
}

const alignments = {
  bottom: "50% 80%",
  "bottom-end": "50% 100%",
  top: "50% 30%",
  "top-end": "50% 0%",
  left: "30% 50%",
  "left-end": "0% 50%",
  right: "80% 50%",
  "right-end": "100% 50%",
};

const typicalPageWidth = 768;

export function PhotoGrid({
  style,
  base,
  width,
  widths,
  heights,
  items,
}: PhotoGridProps) {
  return;
  if (!base.endsWith("/")) base += "/";
  let rows: boolean[][] = [];
  const row = (y: number) => (rows[y] ??= new Array(width).fill(false));
  const anyCollide = (x: number, y: number, w: number, h: number) => {
    for (let yy = 0; yy < h; yy++) {
      const r = row(y + yy);
      for (let xx = 0; xx < w; xx++) if (r[x + xx]) return true;
    }
    return false;
  };

  const firstRowHeight = heights?.[0] ?? 325;

  let x = 0;
  let y = 0;
  let contents = [];
  for (const item of items) {
    const [img, { w = 1, h = 1, align } = {}] = isArray(item) ? item : [item];
    if (w > width) throw new Error(`Item ${img} too wide (${w} > ${width})`);

    while (anyCollide(x, y, w, h)) {
      x += 1;
      if (x > width - w) x = 0, y += 1;
    }
    for (let yy = 0; yy < h; yy++) {
      const r = row(y + yy);
      for (let xx = 0; xx < w; xx++) r[x + xx] = true;
    }

    const info = fileInfo(base, img);
    contents.push(
      <div
        style={{
          ...x === 0 && y === 0 &&
            {
              "aspect-ratio": ((w / width) * typicalPageWidth) / firstRowHeight,
            },
          "grid-area": `${y + 1} / ${x + 1} / ${1 + y + h} / ${1 + x + w}`,
        }}
      >
        <img
          {...info}
          style={{
            "object-position": align
              ? (alignments as Record<string, string>)[align] ?? align
              : "50% 50%",
          }}
        />
      </div>,
    );
  }

  const height = rows.length;
  return (
    <photo-grid
      style={{
        ...style,
        "grid-template-columns": widths
          ? mapSizes(widths).join(" ")
          : `repeat(${width}, 1fr)`,
        "grid-template-rows": heights
          ? heights.map((h) => (h / firstRowHeight) + "fr")
            .concat(new Array(height - heights.length).fill("1fr"))
            .join(" ")
          : `repeat(${height}, 1fr)`,
      }}
    >
      {contents}
    </photo-grid>
  );
}

function fileInfo(base: string, img: string) {
  const filePath = path.join(base, img);
  const file = MediaFile.getByPath(filePath);
  const dimensions = file?.parseDimensions();
  if (!dimensions) {
    throw new Error(`File does not exist or has no dimensions: ${filePath}`);
  }
  const src = "/file" + base + encodeURIComponent(img);
  const alt = path.basename(img, path.extname(img));
  return { src, alt, width: dimensions.width, height: dimensions.height };
}

function mapSizes(sizes: Array<string | number>) {
  return sizes.map(mapSize);
}
function mapSize(size: string | number) {
  return typeof size === "number" ? size + "px" : size;
}

import * as path from "node:path";
